@@ -47,6 +47,7 @@ namespace Tomboy
 
                 static Gdk.Pixbuf note_icon;
                 static Gdk.Pixbuf all_notes_icon;
+                static Gdk.Pixbuf unfiled_notes_icon;
                 static Gdk.Pixbuf notebook_icon;
                 static ArrayList previous_searches;
                 static NoteRecentChanges instance;
@@ -55,6 +56,7 @@ namespace Tomboy
                 {
                         note_icon = GuiUtils.GetIcon ("tomboy-note", 22);
                         all_notes_icon = GuiUtils.GetIcon ("tomboy-all-notes", 22);
+                        unfiled_notes_icon = GuiUtils.GetIcon ("tomboy-unfiled-notes", 22);
                         notebook_icon = GuiUtils.GetIcon ("tomboy-notebook", 22);
                 }
 
@@ -208,6 +210,7 @@ namespace Tomboy
 
                         am ["OpenNoteAction"].Activated += OnOpenNote;
                         am ["DeleteNoteAction"].Activated += OnDeleteNote;
+                        am ["NewNotebookNoteAction"].Activated += OnNewNotebookNote;
                         am ["OpenNotebookTemplateNoteAction"].Activated += OnOpenNotebookTemplateNote;
 			am ["NewNotebookAction"].Activated += OnNewNotebook;
                         am ["DeleteNotebookAction"].Activated += OnDeleteNotebook;
@@ -230,7 +233,7 @@ namespace Tomboy
 
 		Gtk.Widget MakeNotebooksPane ()
 		{
-			notebooksTree = new Notebooks.NotebooksTreeView (Notebooks.NotebookManager.NotebooksWithAllNotesItem);
+			notebooksTree = new Notebooks.NotebooksTreeView (Notebooks.NotebookManager.NotebooksWithSpecialItems);
 			notebooksTree.Selection.Mode = Gtk.SelectionMode.Single;
 			notebooksTree.HeadersVisible = true;
 			notebooksTree.RulesHint = false;
@@ -290,6 +293,7 @@ namespace Tomboy
                         tree.RulesHint = true;
                         tree.RowActivated += OnRowActivated;
                         tree.DragDataGet += OnDragDataGet;
+                        tree.Selection.Mode = Gtk.SelectionMode.Multiple;
                         tree.Selection.Changed += OnSelectionChanged;
                         tree.ButtonPressEvent += OnButtonPressed;
 
@@ -337,14 +341,8 @@ namespace Tomboy
 
                 void UpdateResults ()
                 {
-                        // Restore the currently highlighted note
-                        Note selected_note = null;
-                        Gtk.TreeIter selected_iter;
-                        if (store_sort != null &&
-                                        tree.Selection.GetSelected (out selected_iter)) {
-                                selected_note =
-                                        (Note) store_sort.GetValue (selected_iter, 3 /* note */);
-                        }
+                        // Save the currently selected notes
+                        List<Note> selected_notes = GetSelectedNotes ();
 
                         int sort_column = 2; /* change date */
                         Gtk.SortType sort_type = Gtk.SortType.Descending;
@@ -385,12 +383,12 @@ namespace Tomboy
                         }
 
                         // Restore the previous selection
-                        if (selected_note != null) {
-                                SelectNote (selected_note);
+                        if (selected_notes != null && selected_notes.Count > 0) {
+                        	SelectNotes (selected_notes);
                         }
                 }
 
-                void SelectNote (Note note)
+                void SelectNotes (List<Note> notes)
                 {
                         Gtk.TreeIter iter;
 
@@ -398,14 +396,21 @@ namespace Tomboy
                                 return;
 
                         do {
-                                Note iter_note = (Note) store_sort.GetValue (iter, 3 /* note */);
-                                if (iter_note == note) {
-                                        // Found it!
-                                        tree.Selection.SelectIter (iter);
-                                        break;
-                                }
+                        	Note iter_note = (Note) store_sort.GetValue (iter, 3 /* note */);
+                        	if (notes.IndexOf (iter_note) >= 0) {
+                        		// Found one
+                        		tree.Selection.SelectIter (iter);
+                        		//ScrollToIter (tree, iter);
+                        	}
                         } while (store_sort.IterNext (ref iter));
                 }
+		
+		private void ScrollToIter (Gtk.TreeView tree, Gtk.TreeIter iter)
+		{
+			Gtk.TreePath path = tree.Model.GetPath (iter);
+			if (path != null)
+				tree.ScrollToCell (path, null, false, 0, 0);
+		}
 
 		void PerformSearch()
 		{
@@ -430,7 +435,7 @@ namespace Tomboy
 			
 			// Search using the currently selected notebook
 			Notebooks.Notebook selected_notebook = GetSelectedNotebook ();
-			if (selected_notebook is Notebooks.AllNotesNotebook)
+			if (selected_notebook is Notebooks.SpecialNotebook)
 				selected_notebook = null;
 			
 			IDictionary<Note,int> results =
@@ -549,6 +554,15 @@ namespace Tomboy
                         Tag template_tag = TagManager.GetOrCreateSystemTag (TagManager.TemplateNoteSystemTag);
                         if (note.ContainsTag (template_tag))
                         	return false;
+                        
+                        Notebooks.Notebook selected_notebook = GetSelectedNotebook ();
+                        if (selected_notebook is Notebooks.UnfiledNotesNotebook) {
+                        	// If the note belongs to a notebook, return false
+                        	// since the only notes that should be shown in this
+                        	// case are notes that are unfiled (not in a notebook).
+                        	if (Notebooks.NotebookManager.GetNotebookFromNote (note) != null)
+                        		return false;
+                        }
 
                         bool passes_search_filter = FilterBySearch (note);
                         if (passes_search_filter == false)
@@ -628,60 +642,77 @@ namespace Tomboy
 
                 void OnDragDataGet (object sender, Gtk.DragDataGetArgs args)
                 {
-                        Note note = GetSelectedNote ();
-                        if (note == null)
-                                return;
-
-                        // FIXME: Gtk.SelectionData has no way to get the
-                        //        requested target.
-
-                        args.SelectionData.Set (Gdk.Atom.Intern ("text/uri-list", false),
-                                                8,
-                                                Encoding.UTF8.GetBytes (note.Uri));
-
-                        args.SelectionData.Text = note.Title;
+                	List<Note> selected_notes = GetSelectedNotes ();
+                	if (selected_notes == null || selected_notes.Count == 0)
+                		return;
+                	
+                	string uris = string.Empty;
+                	foreach (Note note in selected_notes) {
+                		if (uris != string.Empty)
+                			uris += "\n";
+                		uris += note.Uri;
+                	}
+                	
+                	// FIXME: Gtk.SelectionData has no way to get the
+                	//        requested target.
+                	
+                	args.SelectionData.Set (Gdk.Atom.Intern ("text/uri-list", false),
+                							8,
+                							Encoding.UTF8.GetBytes (uris));
+                	
+                	if (selected_notes.Count == 1)
+                		args.SelectionData.Text = selected_notes [0].Title;
+                	else
+                		args.SelectionData.Text = Catalog.GetString ("Notes");
                 }
 
                 void OnSelectionChanged (object sender, EventArgs args)
                 {
-                        Note note = GetSelectedNote ();
-                        if (note != null) {
-                                Tomboy.ActionManager ["OpenNoteAction"].Sensitive = true;
-                                Tomboy.ActionManager ["DeleteNoteAction"].Sensitive = true;
-                        } else {
-                                Tomboy.ActionManager ["OpenNoteAction"].Sensitive = false;
-                                Tomboy.ActionManager ["DeleteNoteAction"].Sensitive = false;
-                        }
+                	List<Note> selected_notes = GetSelectedNotes ();
+                	if (selected_notes == null || selected_notes.Count == 0) {
+						Tomboy.ActionManager ["OpenNoteAction"].Sensitive = false;
+						Tomboy.ActionManager ["DeleteNoteAction"].Sensitive = false;
+                	} else if (selected_notes.Count == 1) {
+						Tomboy.ActionManager ["OpenNoteAction"].Sensitive = true;
+						Tomboy.ActionManager ["DeleteNoteAction"].Sensitive = true;
+                	} else {
+                		// Many notes are selected
+						Tomboy.ActionManager ["OpenNoteAction"].Sensitive = false;
+						Tomboy.ActionManager ["DeleteNoteAction"].Sensitive = true;
+                	}
                 }
 
-                [GLib.ConnectBefore]
+				[GLib.ConnectBefore]
                 void OnButtonPressed (object sender, Gtk.ButtonPressEventArgs args)
                 {
-                        switch (args.Event.Button) {
+                	Gtk.TreePath path = null;
+                	Gtk.TreeViewColumn column = null;
+					Gtk.TreeSelection selection = tree.Selection;
+					if (selection.CountSelectedRows () == 0)
+						return;
+                	
+                	switch (args.Event.Button) {
                         case 3: // third mouse button (right-click)
-                                Gtk.TreePath path = null;
-                                Gtk.TreeViewColumn column = null;
-
                                 if (tree.GetPathAtPos ((int) args.Event.X,
                                                        (int) args.Event.Y,
                                                        out path,
                                                        out column) == false)
                                         break;
 
-                                Gtk.TreeSelection selection = tree.Selection;
-                                if (selection.CountSelectedRows () == 0)
-                                        break;
-
 		                        Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
                                             "/MainWindowContextMenu") as Gtk.Menu;
                                 PopupContextMenuAtLocation (menu,
                                 							(int) args.Event.X,
                                                             (int) args.Event.Y);
-
+                                
+                                // Return true so that the base handler won't
+                                // run, which causes the selection to change
+                                // to the row that was right-clicked.
+                                args.RetVal = true;
                                 break;
                         }
                 }
-
+                
                 void PopupContextMenuAtLocation (Gtk.Menu menu, int x, int y)
                 {
                         menu.ShowAll ();
@@ -757,33 +788,41 @@ namespace Tomboy
                         y += widget_y;
                 }
 
-                Note GetSelectedNote ()
-                {
-                        Gtk.TreeModel model;
-                        Gtk.TreeIter iter;
-
-                        if (!tree.Selection.GetSelected (out model, out iter))
-                                return null;
-
-                        return (Note) model.GetValue (iter, 3 /* note */);
-                }
+				List<Note> GetSelectedNotes ()
+				{
+					Gtk.TreeModel model;
+					Gtk.TreeIter iter;
+					
+					List<Note> selected_notes = new List<Note> ();
+					
+					Gtk.TreePath [] selected_rows =
+						tree.Selection.GetSelectedRows (out model);
+					foreach (Gtk.TreePath path in selected_rows) {
+						if (model.GetIter (out iter, path)) {
+							Note note = model.GetValue (iter, 3 /* note */) as Note;
+							selected_notes.Add (note);
+						}
+					}
+					
+					return selected_notes;
+				}
 
                 void OnOpenNote (object sender, EventArgs args)
                 {
-                        Note note = GetSelectedNote ();
-                        if (note == null)
-                                return;
-
-                        note.Window.Present ();
+                	List<Note> selected_notes = GetSelectedNotes ();
+                	if (selected_notes == null || selected_notes.Count != 1)
+                		return;
+                	
+                	selected_notes [0].Window.Present ();
                 }
 
                 void OnDeleteNote (object sender, EventArgs args)
                 {
-                        Note note = GetSelectedNote ();
-                        if (note == null)
-                                return;
-
-                        NoteUtils.ShowDeletionDialog (note, this);
+                	List<Note> selected_notes = GetSelectedNotes ();
+                	if (selected_notes == null || selected_notes.Count == 0)
+                		return;
+                	
+               		NoteUtils.ShowDeletionDialog (selected_notes, this);
                 }
 
                 void OnCloseWindow (object sender, EventArgs args)
@@ -806,6 +845,7 @@ namespace Tomboy
                                 am ["DeleteNoteAction"].Activated -= OnDeleteNote;
 				am ["NewNotebookAction"].Activated -= OnNewNotebook;
 				am ["DeleteNotebookAction"].Activated -= OnDeleteNotebook;
+                am ["NewNotebookNoteAction"].Activated -= OnNewNotebookNote;
 				am ["OpenNotebookTemplateNoteAction"].Activated -= OnOpenNotebookTemplateNote;
                                 am ["CloseWindowAction"].Activated -= OnCloseWindow;
                         }
@@ -833,8 +873,8 @@ namespace Tomboy
                                 break;
                         case Gdk.Key.Menu:
                                 // Pop up the context menu if a note is selected
-                                Note note = GetSelectedNote ();
-                                if (note != null) {
+                                List<Note> selected_notes = GetSelectedNotes ();
+                                if (selected_notes != null && selected_notes.Count > 0) {
 			                        Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
                                                 "/MainWindowContextMenu") as Gtk.Menu;
                                     PopupContextMenuAtLocation (menu, 0, 0);
@@ -1015,6 +1055,8 @@ namespace Tomboy
 			Gtk.CellRendererPixbuf crp = renderer as Gtk.CellRendererPixbuf;
 			if (notebook is Notebooks.AllNotesNotebook) {
 				crp.Pixbuf = all_notes_icon;
+			} else if (notebook is Notebooks.UnfiledNotesNotebook) {
+				crp.Pixbuf = unfiled_notes_icon;
 			} else {
 				crp.Pixbuf = notebook_icon;
 			}
@@ -1034,8 +1076,8 @@ namespace Tomboy
 			
 			crt.Text = notebook.Name;
 
-			if (notebook is Notebooks.AllNotesNotebook) {
-				// Bold the All Notes item
+			if (notebook is Notebooks.SpecialNotebook) {
+				// Bold the "Special" Notebooks
 				crt.Markup = string.Format ("<span weight=\"bold\">{0}</span>", notebook.Name);
 			} else {
 				crt.Text = notebook.Name;
@@ -1049,7 +1091,7 @@ namespace Tomboy
 				// Clear out the currently selected tags so that no notebook is selected
 				selected_tags.Clear ();
                 Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = false;
-				Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = false;
+
 				
 				// Select the "All Notes" item without causing
 				// this handler to be called again
@@ -1060,11 +1102,9 @@ namespace Tomboy
 				selected_tags.Clear ();
 				if (notebook.Tag != null)
 					selected_tags.Add (notebook.Tag, notebook.Tag);
-				if (notebook is Notebooks.AllNotesNotebook) {
-	                Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = true;
+				if (notebook is Notebooks.SpecialNotebook) {
 					Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = false;
 				} else {
-	                Tomboy.ActionManager ["OpenNotebookTemplateNoteAction"].Sensitive = true;
 					Tomboy.ActionManager ["DeleteNotebookAction"].Sensitive = true;
 				}
 			}
@@ -1092,6 +1132,30 @@ namespace Tomboy
 			OnOpenNotebookTemplateNote (sender, EventArgs.Empty);
 		}
 		
+		private void OnNewNotebookNote (object sender, EventArgs args)
+		{
+			Notebooks.Notebook notebook = GetSelectedNotebook ();
+			if (notebook == null || notebook is Notebooks.SpecialNotebook) {
+				// Just create a standard note (not in a notebook)
+				Tomboy.ActionManager ["NewNoteAction"].Activate ();
+				return;
+			}
+			
+			// Look for the template note and create a new note
+			Note templateNote = notebook.GetTemplateNote ();
+			Note note;
+			
+			note = manager.Create ();
+			if (templateNote != null) {
+				// Use the body from the template note
+				string xmlContent = templateNote.XmlContent.Replace (templateNote.Title, note.Title);
+				note.XmlContent = xmlContent;
+			}
+			
+			note.AddTag (notebook.Tag);
+			note.Window.Show ();
+		}
+		
 		private void OnOpenNotebookTemplateNote (object sender, EventArgs args)
 		{
 			Notebooks.Notebook notebook = GetSelectedNotebook ();
@@ -1172,7 +1236,7 @@ namespace Tomboy
 				case Gdk.Key.Menu:
 					// Pop up the context menu if a notebook is selected
 					Notebooks.Notebook notebook = GetSelectedNotebook ();
-					if (notebook == null || notebook is Notebooks.AllNotesNotebook)
+					if (notebook == null || notebook is Notebooks.SpecialNotebook)
 						return; // Don't pop open a submenu
 					
 					Gtk.Menu menu = Tomboy.ActionManager.GetWidget (
