@@ -9,41 +9,41 @@ namespace Tomboy.Sync
 	{
 		private List<string> updatedNotes;
 		private List<string> deletedNotes;
-		
+
 		private string serverId;
-		
+
 		private string serverPath;
 		private string notePath;
 		private string lockPath;
 		private string manifestPath;
-		
+
 		private int newRevision;
 		private string newRevisionPath;
-		
+
 		private static DateTime initialSyncAttempt = DateTime.MinValue;
 		private static string lastSyncLockHash = string.Empty;
 		InterruptableTimeout lockTimeout;
 		SyncLockInfo syncLock;
-		
+
 		public FileSystemSyncServer (string localSyncPath)
 		{
 			serverPath = localSyncPath;
-			
+
 			if (!Directory.Exists (serverPath))
 				throw new DirectoryNotFoundException (serverPath);
-			
+
 			notePath = Tomboy.DefaultNoteManager.NoteDirectoryPath;
 			lockPath = Path.Combine (serverPath, "lock");
 			manifestPath = Path.Combine (serverPath, "manifest.xml");
-			
+
 			newRevision = LatestRevision + 1;
 			newRevisionPath = GetRevisionDirPath (newRevision);
-			
+
 			lockTimeout = new InterruptableTimeout ();
 			lockTimeout.Timeout += LockTimeout;
 			syncLock = new SyncLockInfo ();
 		}
-		
+
 		public virtual void UploadNotes (IList<Note> notes)
 		{
 			if (Directory.Exists (newRevisionPath) == false) {
@@ -78,13 +78,13 @@ namespace Tomboy.Sync
 		public IList<string> GetAllNoteUUIDs ()
 		{
 			List<string> noteUUIDs = new List<string> ();
-			
+
 			if (IsValidXmlFile (manifestPath)) {
 				// TODO: Permission errors
 				using (FileStream fs = new FileStream (manifestPath, FileMode.Open)) {
 					XmlDocument doc = new XmlDocument ();
 					doc.Load (fs);
-					
+
 					XmlNodeList noteIds = doc.SelectNodes ("//note/@id");
 					Logger.Debug ("GetAllNoteUUIDs has {0} notes", noteIds.Count);
 					foreach (XmlNode idNode in noteIds) {
@@ -92,10 +92,10 @@ namespace Tomboy.Sync
 					}
 				}
 			}
-			
+
 			return noteUUIDs;
 		}
-		
+
 		public virtual IDictionary<string, NoteUpdate> GetNoteUpdatesSince (int revision)
 		{
 			Dictionary<string, NoteUpdate> noteUpdates = new Dictionary<string, NoteUpdate> ();
@@ -111,17 +111,17 @@ namespace Tomboy.Sync
 					}
 				} catch {}
 			}
-			
-			if (IsValidXmlFile (manifestPath)) {
+
+		if (IsValidXmlFile (manifestPath)) {
 				// TODO: Permissions errors
 				using (FileStream fs = new FileStream (manifestPath, FileMode.Open)) {
 					XmlDocument doc = new XmlDocument ();
 					doc.Load (fs);
-					
+
 					string xpath =
-						string.Format ("//note[@rev > {0}]", revision.ToString ());
+					        string.Format ("//note[@rev > {0}]", revision.ToString ());
 					XmlNodeList noteNodes = doc.SelectNodes (xpath);
-	Logger.Debug ("GetNoteUpdatesSince xpath returned {0} nodes", noteNodes.Count);
+					Logger.Debug ("GetNoteUpdatesSince xpath returned {0} nodes", noteNodes.Count);
 					foreach (XmlNode node in noteNodes) {
 						string id = node.SelectSingleNode ("@id").InnerText;
 						int rev = Int32.Parse (node.SelectSingleNode ("@rev").InnerText);
@@ -131,7 +131,7 @@ namespace Tomboy.Sync
 							string serverNotePath = Path.Combine (revDir, id + ".note");
 							string noteTempPath = Path.Combine (tempPath, id + ".note");
 							File.Copy (serverNotePath, noteTempPath, true);
-							
+
 							// Get the title, contents, etc.
 							string noteTitle = string.Empty;
 							string noteXml = null;
@@ -144,7 +144,7 @@ namespace Tomboy.Sync
 					}
 				}
 			}
-			
+
 			Logger.Debug ("GetNoteUpdatesSince ({0}) returning: {1}", revision, noteUpdates.Count);
 			return noteUpdates;
 		}
@@ -179,16 +179,16 @@ namespace Tomboy.Sync
 							return false;
 						}
 					}
-					
+
 					// Cleanup Old Sync Lock!
 					CleanupOldSync (currentSyncLock);
 				}
 			}
-			
+
 			// Reset the initialSyncAttempt
 			initialSyncAttempt = DateTime.MinValue;
 			lastSyncLockHash = string.Empty;
-			
+
 			// Create a new lock file so other clients know another client is
 			// actively synchronizing right now.
 			syncLock.RenewCount = 0;
@@ -198,29 +198,29 @@ namespace Tomboy.Sync
 			// out some other way to automatically update the lock file.
 			// Reset the timer to 20 seconds sooner than the sync lock duration
 			lockTimeout.Reset ((uint)syncLock.Duration.TotalMilliseconds - 20000);
-			
+
 			updatedNotes = new List<string> ();
 			deletedNotes = new List<string> ();
-			
+
 			return true;
 		}
 
 		public virtual bool CommitSyncTransaction ()
 		{
 			bool commitSucceeded = false;
-			
+
 			if (updatedNotes.Count > 0 || deletedNotes.Count > 0)
 			{
 				// TODO: error-checking, etc
 				string manifestFilePath = Path.Combine (newRevisionPath,
-														"manifest.xml");
+				                                        "manifest.xml");
 				if (!Directory.Exists (newRevisionPath))
 				{
 					DirectoryInfo info = Directory.CreateDirectory (newRevisionPath);
 					AdjustPermissions (info.Parent.FullName);
 					AdjustPermissions (newRevisionPath);
 				}
-				
+
 				XmlNodeList noteNodes = null;
 				if (IsValidXmlFile (manifestPath) == true) {
 					using (FileStream fs = new FileStream (manifestPath, FileMode.Open)) {
@@ -235,52 +235,52 @@ namespace Tomboy.Sync
 						noteNodes = doc.SelectNodes ("//note");
 					}
 				}
-				
+
 				// Write out the new manifest file
 				XmlTextWriter xml = new XmlTextWriter (manifestFilePath, System.Text.Encoding.UTF8);
 				try {
-				xml.Formatting = Formatting.Indented;
+					xml.Formatting = Formatting.Indented;
 
-				xml.WriteStartDocument ();
-				xml.WriteStartElement (null, "sync", null);
-				xml.WriteAttributeString ("revision", newRevision.ToString ());
-				xml.WriteAttributeString ("server-id", serverId);
+					xml.WriteStartDocument ();
+					xml.WriteStartElement (null, "sync", null);
+					xml.WriteAttributeString ("revision", newRevision.ToString ());
+					xml.WriteAttributeString ("server-id", serverId);
+
+					foreach (XmlNode node in noteNodes) {
+						string id = node.SelectSingleNode ("@id").InnerText;
+						string rev = node.SelectSingleNode ("@rev").InnerText;
+
+						// Don't write out deleted notes
+						if (deletedNotes.Contains (id))
+							continue;
+
+						// Skip updated notes, we'll update them in a sec
+						if (updatedNotes.Contains (id))
+							continue;
+
+						xml.WriteStartElement (null, "note", null);
+						xml.WriteAttributeString ("id", id);
+						xml.WriteAttributeString ("rev", rev);
+						xml.WriteEndElement ();
+					}
+
+					// Write out all the updated notes
+					foreach (string uuid in updatedNotes) {
+						xml.WriteStartElement (null, "note", null);
+						xml.WriteAttributeString ("id", uuid);
+						xml.WriteAttributeString ("rev", newRevision.ToString ());
+						xml.WriteEndElement ();
+					}
 
-				foreach (XmlNode node in noteNodes) {
-					string id = node.SelectSingleNode ("@id").InnerText;
-					string rev = node.SelectSingleNode ("@rev").InnerText;
-					
-					// Don't write out deleted notes
-					if (deletedNotes.Contains (id))
-						continue;
-					
-					// Skip updated notes, we'll update them in a sec
-					if (updatedNotes.Contains (id))
-						continue;
-					
-					xml.WriteStartElement (null, "note", null);
-					xml.WriteAttributeString ("id", id);
-					xml.WriteAttributeString ("rev", rev);
 					xml.WriteEndElement ();
-				}
-				
-				// Write out all the updated notes
-				foreach (string uuid in updatedNotes) {
-					xml.WriteStartElement (null, "note", null);
-					xml.WriteAttributeString ("id", uuid);
-					xml.WriteAttributeString ("rev", newRevision.ToString ());
-					xml.WriteEndElement ();
-				}
-				
-				xml.WriteEndElement ();
-				xml.WriteEndDocument ();
+					xml.WriteEndDocument ();
 				} finally {
 					xml.Close ();
 				}
-				
+
 				AdjustPermissions (manifestFilePath);
 
-				
+
 				// Rename original /manifest.xml to /manifest.xml.old
 				string oldManifestPath = manifestPath + ".old";
 				if (File.Exists (manifestPath) == true) {
@@ -289,50 +289,56 @@ namespace Tomboy.Sync
 					}
 					File.Move (manifestPath, oldManifestPath);
 				}
-				
+
 				// * * * Begin Cleanup Code * * *
 				// TODO: Consider completely discarding cleanup code, in favor
-				//       of period thorough server consistency checks (say every 30 revs)
+				//       of periodic thorough server consistency checks (say every 30 revs).
 				//       Even if we do continue providing some cleanup, consistency
 				//       checks should be implemented.
-				// TODO: If this cleanup fails, transaction should not be considered a failure!
-				
+
 				// Copy the /${parent}/${rev}/manifest.xml -> /manifest.xml
 				File.Copy (manifestFilePath, manifestPath);
 				AdjustPermissions (manifestPath);
-				
-				// Delete /manifest.xml.old
-				if (File.Exists (oldManifestPath))
-					File.Delete (oldManifestPath);
-				
-				string oldManifestFilePath = Path.Combine (GetRevisionDirPath (newRevision - 1),
-				                                           "manifest.xml");
-				if (File.Exists (oldManifestFilePath)) {
-					// TODO: Do step #8 as described in http://bugzilla.gnome.org/show_bug.cgi?id=321037#c17
-					// Like this?
-					FileInfo oldManifestFilePathInfo = new FileInfo (oldManifestFilePath);
-					foreach (FileInfo file in oldManifestFilePathInfo.Directory.GetFiles ()) {
-						string fileGuid = Path.GetFileNameWithoutExtension (file.Name);
-						if (deletedNotes.Contains (fileGuid) ||
-						    updatedNotes.Contains (fileGuid))
-							File.Delete (file.FullName);
-						// TODO: Need to check *all* revision dirs, not just previous (duh)
-						//       Should be a way to cache this from checking earlier.
+
+				try {
+					// Delete /manifest.xml.old
+					if (File.Exists (oldManifestPath))
+						File.Delete (oldManifestPath);
+
+					string oldManifestFilePath = Path.Combine (GetRevisionDirPath (newRevision - 1),
+					                             "manifest.xml");
+					if (File.Exists (oldManifestFilePath)) {
+						// TODO: Do step #8 as described in http://bugzilla.gnome.org/show_bug.cgi?id=321037#c17
+						// Like this?
+						FileInfo oldManifestFilePathInfo = new FileInfo (oldManifestFilePath);
+						foreach (FileInfo file in oldManifestFilePathInfo.Directory.GetFiles ()) {
+							string fileGuid = Path.GetFileNameWithoutExtension (file.Name);
+							if (deletedNotes.Contains (fileGuid) ||
+							                updatedNotes.Contains (fileGuid))
+								File.Delete (file.FullName);
+							// TODO: Need to check *all* revision dirs, not just previous (duh)
+							//       Should be a way to cache this from checking earlier.
+						}
+
+						// TODO: Leaving old empty dir for now.  Some stuff is probably easier
+						//       when you can guarantee the existence of each intermediate directory?
+
 					}
-					
-					// TODO: Leaving old empty dir for now.  Some stuff is probably easier
-					//       when you can guarantee the existence of each intermediate directory?
-	
+				} catch (Exception e) {
+					Logger.Error ("Exception during server cleanup while committing. " +
+					              "Server integrity is OK, but there may be some excess " +
+					              "files floating around.  Here's the error:\n" +
+					              e.Message);
 				}
 				// * * * End Cleanup Code * * *
 			}
-			
+
 			lockTimeout.Cancel ();
 			File.Delete (lockPath);// TODO: Errors?
 			commitSucceeded = true;// TODO: When return false?
 			return commitSucceeded;
 		}
-		
+
 		// TODO: Return false if this is a bad time to cancel sync?
 		public bool CancelSyncTransaction ()
 		{
@@ -340,7 +346,7 @@ namespace Tomboy.Sync
 			File.Delete (lockPath);
 			return true;
 		}
-		
+
 		public virtual int LatestRevision
 		{
 			get
@@ -357,7 +363,7 @@ namespace Tomboy.Sync
 							latestRev = Int32.Parse (latestRevStr);
 					}
 				}
-				
+
 				bool foundValidManifest = false;
 				while (!foundValidManifest)
 				{
@@ -371,18 +377,18 @@ namespace Tomboy.Sync
 							} catch {}
 						}
 
-						if (latestRevDir >= 0) {
+					if (latestRevDir >= 0) {
 							foreach (string revDir in Directory.GetDirectories (
-										Path.Combine (serverPath, latestRevDir.ToString ()))) {
+							                 Path.Combine (serverPath, latestRevDir.ToString ()))) {
 								try {
 									int currentRev = Int32.Parse (revDir);
 									if (currentRev > latestRev)
 										latestRev = currentRev;
 								} catch {}
 							}
-						}
-						
-						if (latestRev >= 0) {
+					}
+
+					if (latestRev >= 0) {
 							// Validate that the manifest file inside the revision is valid
 							// TODO: Should we create the /manifest.xml file with a valid one?
 							string revDirPath = GetRevisionDirPath (latestRev);
@@ -399,16 +405,16 @@ namespace Tomboy.Sync
 					} else
 						foundValidManifest = true;
 				}
-				
+
 				return latestRev;
 			}
 		}
-		
+
 		public virtual SyncLockInfo CurrentSyncLock
 		{
 			get {
 				SyncLockInfo syncLockInfo = new SyncLockInfo ();
-				
+
 				if (IsValidXmlFile (lockPath)) {
 					// TODO: Permissions errors
 					using (FileStream fs = new FileStream (lockPath, FileMode.Open)) {
@@ -421,25 +427,25 @@ namespace Tomboy.Sync
 							string transaction_id_txt = node.InnerText;
 							syncLockInfo.TransactionId = transaction_id_txt;
 						}
-						
+
 						node = doc.SelectSingleNode ("//client-id/text ()");
 						if (node != null) {
 							string client_id_txt = node.InnerText;
 							syncLockInfo.ClientId = client_id_txt;
 						}
-						
+
 						node = doc.SelectSingleNode ("//renew-count/text ()");
 						if (node != null) {
 							string renew_txt = node.InnerText;
 							syncLockInfo.RenewCount = Int32.Parse (renew_txt);
 						}
-						
+
 						node = doc.SelectSingleNode ("//lock-expiration-duration/text ()");
 						if (node != null) {
 							string span_txt = node.InnerText;
 							syncLockInfo.Duration = TimeSpan.Parse (span_txt);
 						}
-						
+
 						node = doc.SelectSingleNode ("//revision/text ()");
 						if (node != null) {
 							string revision_txt = node.InnerText;
@@ -447,17 +453,17 @@ namespace Tomboy.Sync
 						}
 					}
 				}
-				
+
 				return syncLockInfo;
 			}
 		}
-		
+
 		public virtual string Id
 		{
 			get
 			{
 				serverId = null;
-				
+
 				// Attempt to read from manifest file first
 				if (IsValidXmlFile (manifestPath)) {
 					using (FileStream fs = new FileStream (manifestPath, FileMode.Open)) {
@@ -469,63 +475,63 @@ namespace Tomboy.Sync
 							serverId = serverIdNode.InnerText;
 					}
 				}
-				
+
 				// Generate a new ID if there isn't already one
 				if (serverId == null)
 					serverId = System.Guid.NewGuid ().ToString ();
-				
+
 				return serverId;
 			}
 		}
-		
+
 		#region Private Methods
-		
+
 		// NOTE: Assumes serverPath is set
 		private string GetRevisionDirPath (int rev)
 		{
 			return Path.Combine (
-			                     Path.Combine (serverPath, (rev/100).ToString ()),
-			                     rev.ToString ());
+			               Path.Combine (serverPath, (rev/100).ToString ()),
+			               rev.ToString ());
 		}
-		
+
 		private void UpdateLockFile (SyncLockInfo syncLockInfo)
 		{
 			XmlTextWriter xml = new XmlTextWriter (lockPath, System.Text.Encoding.UTF8);
 			try {
-			xml.Formatting = Formatting.Indented;
+				xml.Formatting = Formatting.Indented;
 
-			xml.WriteStartDocument ();
-			xml.WriteStartElement (null, "lock", null);
-			
-			xml.WriteStartElement (null, "transaction-id", null);
-			xml.WriteString (syncLockInfo.TransactionId);
-			xml.WriteEndElement ();
-			
-			xml.WriteStartElement (null, "client-id", null);
-			xml.WriteString (syncLockInfo.ClientId);
-			xml.WriteEndElement ();
-			
-			xml.WriteStartElement (null, "renew-count", null);
-			xml.WriteString (string.Format ("{0}", syncLockInfo.RenewCount));
-			xml.WriteEndElement ();
-			
-			xml.WriteStartElement (null, "lock-expiration-duration", null);
-			xml.WriteString (syncLockInfo.Duration.ToString ());
-			xml.WriteEndElement ();
-			
-			xml.WriteStartElement (null, "revision", null);
-			xml.WriteString (syncLockInfo.Revision.ToString ());
-			xml.WriteEndElement ();
-			
-			xml.WriteEndElement ();
-			xml.WriteEndDocument ();
+				xml.WriteStartDocument ();
+				xml.WriteStartElement (null, "lock", null);
+
+				xml.WriteStartElement (null, "transaction-id", null);
+				xml.WriteString (syncLockInfo.TransactionId);
+				xml.WriteEndElement ();
+
+				xml.WriteStartElement (null, "client-id", null);
+				xml.WriteString (syncLockInfo.ClientId);
+				xml.WriteEndElement ();
+
+				xml.WriteStartElement (null, "renew-count", null);
+				xml.WriteString (string.Format ("{0}", syncLockInfo.RenewCount));
+				xml.WriteEndElement ();
+
+				xml.WriteStartElement (null, "lock-expiration-duration", null);
+				xml.WriteString (syncLockInfo.Duration.ToString ());
+				xml.WriteEndElement ();
+
+				xml.WriteStartElement (null, "revision", null);
+				xml.WriteString (syncLockInfo.Revision.ToString ());
+				xml.WriteEndElement ();
+
+				xml.WriteEndElement ();
+				xml.WriteEndDocument ();
 			} finally {
 				xml.Close ();
 			}
-			
+
 			AdjustPermissions (lockPath);
 		}
-		
+
 		/// <summary>
 		/// This method is used when the sync lock file is determined to be out
 		/// of date.  It will check to see if the manifest.xml file exists and
@@ -543,10 +549,10 @@ namespace Tomboy.Sync
 				for (; rev >= 0; rev--) {
 					string revParentPath = GetRevisionDirPath (rev);
 					string manPath = Path.Combine (revParentPath, "manifest.xml");
-					
+
 					if (IsValidXmlFile (manPath) == false)
 						continue;
-					
+
 					// Restore a valid manifest path
 					File.Copy (manPath, manifestPath, true);
 					break;
@@ -561,7 +567,7 @@ namespace Tomboy.Sync
 				Logger.Warn ("Error deleting the old sync lock \"{0}\": {1}", lockPath, e.Message);
 			}
 		}
-		
+
 		/// <summary>
 		/// Check that xmlFilePath points to an existing valid XML file.
 		/// This is done by ensuring that an XmlDocument can be created from
@@ -572,7 +578,7 @@ namespace Tomboy.Sync
 			// Check that file exists
 			if (!File.Exists (xmlFilePath))
 				return false;
-			
+
 			// TODO: Permissions errors
 			// Attempt to load the file and parse it as XML
 			using (FileStream fs = new FileStream (xmlFilePath, FileMode.Open)) {
@@ -581,19 +587,19 @@ namespace Tomboy.Sync
 					// TODO: Make this be a validating XML reader.  Not sure if it's validating yet.
 					doc.Load (fs);
 				} catch {
-					return false;
-				}
+				return false;
 			}
-			
-			return true;
 		}
-		
-		private void AdjustPermissions (string path)
+
+		return true;
+	}
+
+	private void AdjustPermissions (string path)
 		{
 			Mono.Unix.Native.Syscall.chmod (path, Mono.Unix.Native.FilePermissions.ACCESSPERMS);
 		}
 		#endregion // Private Methods
-		
+
 		#region Private Event Handlers
 		private void LockTimeout (object sender, EventArgs args)
 		{
